
#include <stdio.h>
#include <stdlib.h>
#include "trees.h"
#include <string.h>
#include <signal.h>
#include "stupid.h"
#include "heuristic.h"

move_t ( * enemy ) ( move_t );
const char *prefix = "FroMTreeLIB:";
const char *enprefix = "ENeMY!";

static void die( char *message ) {
	fprintf( stderr, "%s\n", message );
	printf( "0\n" );
	exit( 1 );
}

static void internal( char *message ) {
	char *pr = "FIXME (internal error):";
	char *x = malloc( strlen( message ) + strlen( pr ) + 1 );
	strcpy( x, pr );
	strcpy( x + strlen( pr ), message );
	die( x );
}

static char indexes[ 256 ];
static char decos[] = "ABDS";

static void doMove( node_t *node, char deco, void ( * report )( char *message ) ) {
	if( node->deco != 5 )
		report( "Trying to overwrite decoration" );
	if( !node->avail[ ( unsigned ) deco ] )
		report( "To jail you go, for ilegally decorating a tree" );
	node->deco = deco;
	memset( node->avail, 0, sizeof node->avail );
	if( ! -- tree.rest ) {
		fprintf( stderr, "Lets take a photo, it is decorated\n" );
		printf( "0\n" );
		exit( 0 );
	}
	for( int i = 0; i < node->ncount; ++ i ) {
		if( tree.nodes[ node->neighbors[ i ] ].avail[ ( unsigned ) deco ] ) {
			tree.nodes[ node->neighbors[ i ] ].avail[ ( unsigned ) deco ] = 0;
			int cnt = 0;
			for( int j = 0; j < 4; ++ j )
				if( tree.nodes[ node->neighbors[ i ] ].avail[ j ] )
					cnt ++;
			if( !cnt )
				die( "Undecorable point" );
		}
	}
}

int main( int argc, char *argv[] ) {
	signal(SIGPIPE, SIG_IGN);
	if( argc < 3 ) {
		fprintf( stderr, "Usage: judge opponent input\n" );
		return 2;
	}
	memset( indexes, 5, sizeof indexes );
	for( int i = 0; i < 4; ++ i )
		indexes[ ( unsigned ) decos[ i ] ] = i;
	loadTree( argv[ 2 ] );
	switch( argv[ 1 ][ 0 ] ) {
		case 's':
			enemy = stupidMove;
			break;
		case 'h':
			enemy = heu_move;
			heu_init();
			break;
		default:
			fprintf( stderr, "Unknown enemy\n" );
			exit( 2 );
	}
	char line[ 1024 ];
	for(;;) {
		if( !fgets( line, 1024, stdin ) )
			die( "Input ended (dead program?)" );
		if( !strncmp( line, enprefix, strlen( enprefix ) ) )
			die( "It is your turn, not of your enemy" );
		if( strncmp( line, prefix, strlen( prefix ) ) ) {
			die( "Odd input, missing magic prefix (writing to stdout?)" );
		}
		char *l = line + strlen( prefix );
		while( *l && ( ( *l < '0' ) || ( *l > '9' ) ) )
			l ++;
		if( !l )
			die( "Invalid input, no point number" );
		int node = atoi( l );
		while( *l && ( *l != '\'' ) )
			l ++;
		l ++;
		char deco = indexes[ ( unsigned ) *l ];
		if( deco == 5 )
			die( "Invalid decoration" );
		node --;
		if( ( node < 0 ) || ( node >= tree.N ) )
			die( "No such point in the tree" );
		node_t *point = tree.nodes + node;
		doMove( point, deco, die );
		move_t move = { point, deco };
		puts( "1" );
		fflush( stdout );
		move = enemy( move );
		if( !fgets( line, 1024, stdin ) )
			die( "Input ended (dead program?)" );
		if( !strncmp( line, prefix, strlen( prefix ) ) )
			die( "Not your turn!" );
		if( strncmp( line, enprefix, strlen( enprefix ) ) ) {
			die( "Odd input, missing magic prefix (writing to stdout?)" );
		}
		doMove( move.node, move.deco, internal );
		printf( "%d %c\n", move.node->index + 1, decos[ ( unsigned ) move.deco ] );
		fflush( stdout );
	}
}
