
#ifndef TREE_LIB_H
#define TREE_LIB_H

int init( void );

int neighbor_count( int point );
int neighbor( int point, int index );
char decoration( int point );

int enemy( void );
void decorate( int point, char decoration );

#include <stdlib.h>
#include <stdio.h>

#define CHECK( A ) A

static int tree_lib_N = 0, tree_lib_rest;

typedef struct tree_lib_point_t {
	int neighbor_count, *neighbors;
	char decoration;
} tree_lib_point_t;

static int tree_lib_0[] = { 1 },
	   tree_lib_1[] = { 0, 2, 3, 4 },
	   tree_lib_2[] = { 1 },
	   tree_lib_3[] = { 1 },
	   tree_lib_4[] = { 1, 6, 5, 7 },
	   tree_lib_5[] = { 4 },
	   tree_lib_6[] = { 4 },
	   tree_lib_7[] = { 4 };

static tree_lib_point_t tree_lib_points[] = {
	{ 1, tree_lib_0, ' ' },
	{ 4, tree_lib_1, ' ' },
	{ 1, tree_lib_2, ' ' },
	{ 1, tree_lib_3, ' ' },
	{ 4, tree_lib_4, ' ' },
	{ 1, tree_lib_5, ' ' },
	{ 1, tree_lib_6, ' ' },
	{ 1, tree_lib_7, ' ' }
};

static void tree_lib_die( const char *msg ) {
	fprintf( stderr, "%s\n", msg );
	exit( 1 );
}

static int tree_lib_move = 1;

static char tree_lib_types[] = { 'A', 'B', 'D', 'S' };

int init( void ) {
	CHECK( if( tree_lib_N ) tree_lib_die( "init already called" ); )
	tree_lib_N = 8;
	tree_lib_rest = tree_lib_N;
	return( tree_lib_N );
}

int neighbor_count( int point ) {
	CHECK( if( !tree_lib_N ) tree_lib_die( "No init called" ); )
	point --;
	if( ( point < 0 ) || ( point >= tree_lib_N ) )
		return( 0 );
	return( tree_lib_points[ point ].neighbor_count );
}

int neighbor( int point, int index ) {
	CHECK( if( !tree_lib_N ) tree_lib_die( "No init called" ); )
	point --;
	index --;
	if( ( point < 0 ) || ( point >= tree_lib_N ) )
		return( 0 );
	if( ( index < 0 ) || ( index >= tree_lib_points[ point ].neighbor_count ) )
		return( 0 );
	return( tree_lib_points[ point ].neighbors[ index ] + 1 );
}

char decoration( int point ) {
	CHECK( if( !tree_lib_N ) tree_lib_die( "No init called" ); )
	point --;
	if( ( point < 0 ) || ( point >= tree_lib_N ) )
		return( ' ' );
	return( tree_lib_points[ point ].decoration );
}

int enemy( void ) {
	int i, j, k;
	CHECK( if( !tree_lib_N ) tree_lib_die( "No init called" ); )
	if( tree_lib_move ) tree_lib_die( "It is your turn" );
	tree_lib_move = !tree_lib_move;
	for( i = 0; i < tree_lib_N; ++ i ) {
		char avail[] = { 1, 1, 1, 1 };
		if( tree_lib_points[ i ].decoration != ' ' )
			continue;
		for( j = 0; j < tree_lib_points[ i ].neighbor_count; ++ j ) {
			int neigh = tree_lib_points[ i ].neighbors[ j ];
			if( tree_lib_points[ neigh ].decoration == ' ' )
				continue;
			for( k = 0; k < 4; ++ k ) {
				if( tree_lib_points[ neigh ].decoration == tree_lib_types[ k ] ) {
					avail[ k ] = 0;
					break;
				}
			}
		}
		for( k = 0; k < 4; ++ k )
			if( avail[ k ] ) {
				tree_lib_points[ i ].decoration = tree_lib_types[ k ];
				printf( "%d %c\n", i + 1, tree_lib_types[ k ] );
				if( !( -- tree_lib_rest ) ) {
					exit( 0 );
				}
				return( i + 1 );
			}
	}
	tree_lib_die( "Enemy could not move" );
	return 0;//Just to make compiler happy
}

void decorate( int point, char decoration ) {
	int found = 0, i;
	CHECK( if( !tree_lib_N ) tree_lib_die( "No init called" ); )
	if( !tree_lib_move ) tree_lib_die( "Not your move" );
	tree_lib_move = !tree_lib_move;
	point --;
	if( ( point < 0 ) || ( point >= tree_lib_N ) ) tree_lib_die( "No such point" );
	for( i = 0; i < 4; ++ i )
		if( tree_lib_types[ i ] == decoration ) {
			found = 1;
			break;
		}
	if( !found ) tree_lib_die( "Unknown decoration type" );
	for( i = 0; i < tree_lib_points[ point ].neighbor_count; ++ i )
		if( decoration == tree_lib_points[ tree_lib_points[ point ].neighbors[ i ] ].decoration )
			tree_lib_die( "Ilegal decoration" );
	tree_lib_points[ point ].decoration = decoration;
	printf( "%d %c\n", point + 1, decoration );
	if( !( -- tree_lib_rest ) ) {
		exit( 0 );
	}
}
#endif
